@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2010-2012 Infrae. All rights reserved.
+# Copyright (c) 2010-2013 Infrae. All rights reserved.
 # See also LICENSE.txt
 
 import unittest
 
 import infrae.wsgi
-from infrae.wsgi.testing import BrowserLayer, Browser
+from infrae.wsgi.testing import ZopeBrowserLayer, ZopeBrowser
 from infrae.testing import get_event_names
 from zope.security.management import queryInteraction
 
 
-class WSGILayer(BrowserLayer):
+class WSGILayer(ZopeBrowserLayer):
     default_users = {'admin': ['Manager']}
 
 
@@ -23,7 +23,7 @@ class FunctionalTestCase(unittest.TestCase):
     layer = FunctionalLayer
 
     def setUp(self):
-        self.browser = Browser()
+        self.browser = ZopeBrowser()
         self.browser.handleErrors = True
         self.browser.raiseHttpErrors = False
         get_event_names()       # Clear setup events
